import numpy as np
from scipy import linalg
from scipy.cluster.vq import kmeans2
from sklearn.manifold import TSNE
import matplotlib.pyplot as plt

class Idx:
	uid = 0
	tag1 = 1
	tag2 = 2
	tag3 = 3

def load_data(path):
	data = []
	with open(path, "r") as inf:
		for line in inf:
			datum = line.strip().split(u'\u0001')
			data.append(datum)
	return data

def plot_tsne(proj, labels):
	for i in labels:
		print("%d: %s" % (i, labels[i]))

	x = proj[:, 0]
	y = proj[:, 1]
	fig, ax = plt.subplots()
	ax.scatter(x, y)
	for i in range(len(x)):
	    ax.annotate("%d" % i, (x[i],y[i]))
	plt.show()

if __name__ == "__main__":
	path = “fyy/训练集/TrainingData_Task2.txt"
	data = load_data(path)

	# collect data
	tag_set = set()
	occur_sets = dict()
	for datum in data:
		t1 = datum[Idx.tag1]
		t2 = datum[Idx.tag2]
		t3 = datum[Idx.tag3]

		for t in [t1, t2, t3]:
			tag_set.add(t)
			if t not in occur_sets:
				occur_sets[t] = set()
			occur_sets[t].add(t1)
			occur_sets[t].add(t2)
			occur_sets[t].add(t3)

	print("%d tags" % len(tag_set))

	# give each tag an index
	idx = dict()
	idx_reverse = dict()
	i = 0
	for tag in tag_set:
		idx[tag] = i
		idx_reverse[i] = tag
		i += 1

	# build similarity matrix using Jaccard distance
	n = len(tag_set)
	m = np.zeros(shape=(n,n))
	for t1 in tag_set:
		i1 = idx[t1]
		for t2 in tag_set:
			i2 = idx[t2]
			o1 = occur_sets[t1]
			o2 = occur_sets[t2]
			inter = o1.intersection(o2)
			union = o1.union(o2)
			m[i1][i2] = len(inter) / len(union)
	# np.set_printoptions(threshold=np.nan, suppress=2)
	# print(m)

	# find most related top-3 tags for each tag
	for t in tag_set:
		i = idx[t]
		sim = m[i].argsort()[-4:][::-1]
		t1 = idx_reverse[sim[1]]
		t2 = idx_reverse[sim[2]]
		t3 = idx_reverse[sim[3]]
		print("%s: %s, %s, %s" % (t, t1 ,t2, t3))

	# f, _, _ = linalg.svd(m, full_matrices=True)
	X = linalg.sqrtm(m)

	# clustering using k-means
	cluster_num = 3
	centroids, labels = kmeans2(X, cluster_num, minit='points')
	print(labels)

	clusters = dict()
	for i in range(cluster_num):
		clusters[i] = set()

	for tag in tag_set:
		i = idx[tag]
		cluster_idx = labels[i]
		clusters[cluster_idx].add(tag)

	total_distance = 0
	for i in range(cluster_num):
		print(clusters[i])

		center = centroids[i]
		for tag in clusters[i]:
			tag_idx = idx[tag]
			feature = X[tag_idx]
			dist = linalg.norm(feature - center)
			total_distance += dist
	print("avg distance: %f" % (total_distance / n))

	# plot using t-SNE
	model = TSNE(n_components=2, random_state=0, perplexity=30)
	proj = model.fit_transform(X) 
	# plot_tsne(proj, idx_reverse)
